/*
 * This file is part of Sponge, licensed under the MIT License (MIT).
 *
 * Copyright (c) SpongePowered <https://www.spongepowered.org>
 * Copyright (c) contributors
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package org.spongepowered.common.effect.particle;

import com.flowpowered.math.vector.Vector3d;
import com.google.common.collect.Lists;
import net.minecraft.block.Block;
import net.minecraft.block.state.IBlockState;
import net.minecraft.item.Item;
import net.minecraft.item.ItemBlock;
import net.minecraft.network.Packet;
import net.minecraft.network.play.server.S2APacketParticles;
import net.minecraft.util.EnumParticleTypes;
import org.spongepowered.api.block.BlockState;
import org.spongepowered.api.item.ItemType;
import org.spongepowered.api.item.inventory.ItemStack;
import org.spongepowered.api.util.Color;
import org.spongepowered.common.data.type.SpongeNotePitch;
import org.spongepowered.common.mixin.core.block.MixinBlockState;

import java.util.Collections;
import java.util.List;
import java.util.Random;

public final class SpongeParticleHelper {

    /**
     * Gets the list of packets that are needed to spawn the particle effect at
     * the position. This method tries to minimize the amount of packets for
     * better performance and lower bandwidth use.
     *
     * @param effect The particle effect
     * @param position The position
     * @return The packets
     */
    public static List<Packet> toPackets(SpongeParticleEffect effect, Vector3d position) {
        SpongeParticleType type = effect.getType();
        EnumParticleTypes internal = type.getInternalType();

        Vector3d offset = effect.getOffset();

        int count = effect.getCount();
        int[] extra = new int[0];

        float px = (float) position.getX();
        float py = (float) position.getY();
        float pz = (float) position.getZ();

        double ox = offset.getX();
        double oy = offset.getY();
        double oz = offset.getZ();

        // The extra values, normal behavior offsetX, offsetY, offsetZ
        double f0 = 0f;
        double f1 = 0f;
        double f2 = 0f;

        // Depends on behavior
        // Note: If the count > 0 -> speed = 0f else if count = 0 -> speed = 1f

        if (effect instanceof SpongeParticleEffect.Materialized) {
            ItemStack item = ((SpongeParticleEffect.Materialized) effect).getItem().createStack();
            ItemType itemType = item.getItem();

            int id = 0;
            int data = 0;

            if (internal == EnumParticleTypes.ITEM_CRACK) {
                id = Item.itemRegistry.getIDForObject((Item) itemType);
                data = ((net.minecraft.item.ItemStack) item).getItemDamage();
            }

            if (id == 0) {
                return Collections.emptyList();
            }

            extra = new int[] {id, data};
        }

        if (effect instanceof SpongeParticleEffect.Block) {
            IBlockState blockState = (IBlockState) ((SpongeParticleEffect.Block) effect).getBlockState();
            Block block = blockState.getBlock();
            if (internal == EnumParticleTypes.BLOCK_CRACK || internal == EnumParticleTypes.BLOCK_DUST) {
                // Only block types are allowed
                int id = Block.blockRegistry.getIDForObject(block);
                int data = block.damageDropped(blockState);
                extra = new int[] {id, data};
            }
        }

        if (effect instanceof SpongeParticleEffect.Resized) {
            float size = ((SpongeParticleEffect.Resized) effect).getSize();

            // The formula of the large explosion acts strange
            // Client formula: sizeClient = 1 - sizeServer * 0.5
            // The particle effect returns the client value so
            // Server formula: sizeServer = (-sizeClient * 2) + 2
            if (internal == EnumParticleTypes.EXPLOSION_LARGE) {
                size = (-size * 2f) + 2f;
            }

            if (size == 0f) {
                return Lists.<Packet>newArrayList(new S2APacketParticles(internal, true, px, py, pz, (float) ox, (float) oy, (float) oz, 0f, count,
                        extra));
            }

            f0 = size;
        } else if (effect instanceof SpongeParticleEffect.Colored) {
            Color color0 = ((SpongeParticleEffect.Colored) effect).getColor();
            Color color1 = ((SpongeParticleType.Colorable) type).getDefaultColor();

            if (color0.equals(color1)) {
                return Lists.<Packet>newArrayList(new S2APacketParticles(internal, true, px, py, pz, (float) ox, (float) oy, (float) oz, 0f, count,
                        extra));
            }

            f0 = color0.getRed() / 255f;
            f1 = color0.getGreen() / 255f;
            f2 = color0.getBlue() / 255f;

            // If the f0 value 0 is, the redstone will set it automatically to red 255
            if (f0 == 0f && internal == EnumParticleTypes.REDSTONE) {
                f0 = 0.00001f;
            }
        } else if (effect instanceof SpongeParticleEffect.Note) {
            float note = ((SpongeNotePitch) ((SpongeParticleEffect.Note) effect).getNote()).getByteId();

            if (note == 0f) {
                return Lists.<Packet>newArrayList(new S2APacketParticles(internal, true, px, py, pz, (float) ox, (float) oy, (float) oz, 0f,
                        count, extra));
            }

            f0 = note / 24f;
        } else if (type.hasMotion()) {
            Vector3d motion = effect.getMotion();

            double mx = motion.getX();
            double my = motion.getY();
            double mz = motion.getZ();

            // The y value won't work for this effect, if the value isn't 0 the motion won't work
            if (internal == EnumParticleTypes.WATER_SPLASH) {
                my = 0f;
            }

            if (mx == 0f && my == 0f && mz == 0f) {
                return Lists.<Packet>newArrayList(new S2APacketParticles(internal, true, px, py, pz, (float) ox, (float) oy, (float) oz, 0f, count,
                        extra));
            } else {
                f0 = mx;
                f1 = my;
                f2 = mz;
            }
        }

        // Is this check necessary?
        if (f0 == 0f && f1 == 0f && f2 == 0f) {
            return Lists.<Packet>newArrayList(new S2APacketParticles(internal, true, px, py, pz, (float) ox, (float) oy, (float) oz, 0f, count,
                    extra));
        }

        List<Packet> packets = Lists.newArrayList();

        if (ox == 0f && oy == 0f && oz == 0f) {
            for (int i = 0; i < count; i++) {
                packets.add(new S2APacketParticles(internal, true, px, py, pz, (float) f0, (float) f1, (float) f2, 1f, 0, extra));
            }
        } else {
            Random random = new Random();

            for (int i = 0; i < count; i++) {
                double px0 = (px + (random.nextFloat() * 2f - 1f) * ox);
                double py0 = (py + (random.nextFloat() * 2f - 1f) * oy);
                double pz0 = (pz + (random.nextFloat() * 2f - 1f) * oz);

                packets.add(new S2APacketParticles(internal, true, (float) px0, (float) py0, (float) pz0, (float) f0, (float) f1, (float) f2, 1f, 0,
                        extra));
            }
        }

        return packets;
    }

    private SpongeParticleHelper() {
    }
}
